"use strict";

exports.__esModule = true;
exports.resolvePlugin = resolvePlugin;
exports.resolvePreset = resolvePreset;
exports.loadPlugin = loadPlugin;
exports.loadPreset = loadPreset;
exports.loadParser = loadParser;
exports.loadGenerator = loadGenerator;

var _resolve = require("resolve");

var _resolve2 = _interopRequireDefault(_resolve);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function resolvePlugin(pluginName, dirname) {
  var possibleNames = ["babel-plugin-" + pluginName, pluginName];

  return resolveFromPossibleNames(possibleNames, dirname);
}

function resolvePreset(presetName, dirname) {
  var possibleNames = ["babel-preset-" + presetName, presetName];

  var matches = presetName.match(/^(@[^/]+)\/(.+)$/);
  if (matches) {
    var orgName = matches[1],
        presetPath = matches[2];

    possibleNames.push(orgName + "/babel-preset-" + presetPath);
  }

  return resolveFromPossibleNames(possibleNames, dirname);
}

function loadPlugin(name, dirname) {
  var filepath = resolvePlugin(name, dirname);
  if (!filepath) throw new Error("Plugin " + name + " not found relative to " + dirname);

  return {
    filepath: filepath,
    plugin: requireModule(filepath)
  };
}

function loadPreset(name, dirname) {
  var filepath = resolvePreset(name, dirname);
  if (!filepath) throw new Error("Preset " + name + " not found relative to " + dirname);

  return {
    filepath: filepath,
    preset: requireModule(filepath)
  };
}

function loadParser(name, dirname) {
  var filepath = resolveQuiet(name, dirname);
  if (!filepath) throw new Error("Parser " + name + " not found relative to " + dirname);

  var mod = requireModule(filepath);

  if (!mod) {
    throw new Error("Parser " + name + " relative to " + dirname + " does not export an object");
  }
  if (typeof mod.parse !== "function") {
    throw new Error("Parser " + name + " relative to " + dirname + " does not export a .parse function");
  }

  return {
    filepath: filepath,
    parser: mod.parse
  };
}

function loadGenerator(name, dirname) {
  var filepath = resolveQuiet(name, dirname);
  if (!filepath) throw new Error("Generator " + name + " not found relative to " + dirname);

  var mod = requireModule(filepath);

  if (!mod) {
    throw new Error("Generator " + name + " relative to " + dirname + " does not export an object");
  }
  if (typeof mod.print !== "function") {
    throw new Error("Generator " + name + " relative to " + dirname + " does not export a .print function");
  }

  return {
    filepath: filepath,
    generator: mod.print
  };
}

function resolveQuiet(name, dirname) {
  try {
    return _resolve2.default.sync(name, { basedir: dirname });
  } catch (e) {
    if (e.code !== "MODULE_NOT_FOUND" && e.code !== "ENOTDIR") throw e;
  }
  return null;
}

function resolveFromPossibleNames(possibleNames, dirname) {
  for (var _iterator = possibleNames, _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _iterator[Symbol.iterator]();;) {
    var _ref;

    if (_isArray) {
      if (_i >= _iterator.length) break;
      _ref = _iterator[_i++];
    } else {
      _i = _iterator.next();
      if (_i.done) break;
      _ref = _i.value;
    }

    var name = _ref;

    var result = resolveQuiet(name, dirname);
    if (result !== null) return result;
  }
  return null;
}

function requireModule(name) {
  return require(name);
}